//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 1999  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// AboutGUI class - creates a simple about-this-program dialog
//
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.ListResourceBundle;
import java.text.MessageFormat;

/**
 * Creates a simple about-this-program dialog.
 */
public class AboutGUI extends Dialog
		implements ActionListener, WindowListener {
	private boolean inAnApplet = true;

	// message catalog information
	private ListResourceBundle	messages	= null;

	// GUI components
	private Label		about_label;
	private Button		ok_button;
	private Frame		parent;

	/**
	 * Initializes the whole GUI set.
	 */
	public AboutGUI( boolean inAnApplet, Frame frame, boolean flag,
			Locale locale ) {
		super( frame, flag );
		this.addWindowListener( this );
		parent		= frame;
		this.inAnApplet	= inAnApplet;

		// find the resource bundle for the localized messages
		messages = (ListResourceBundle)
				(ResourceBundle.getBundle(
					"MessageResource", locale ));

		// find the text array from the message catalog
		String about_text[] =
				messages.getStringArray( "AboutGUI.about" );

		// create GUI components
		setTitle( messages.getString( "AboutGUI.title" ));
		setLayout( new BorderLayout() );

		// create the labels
		Panel label_panel = new Panel();
		label_panel.setLayout( new GridLayout( 0, 1 ));
		for ( int index = 0; index < about_text.length; index++ ) {
			label_panel.add( new Label( about_text[ index ],
					Label.CENTER ));
		}
		add( "Center", label_panel );

		Panel button_panel = new Panel();
		button_panel.setLayout( new FlowLayout() );
		ok_button = new Button( messages.getString( "AboutGUI.okbutton" ));
		ok_button.addActionListener( this );
		button_panel.add ( ok_button );
		add( "South", button_panel );
	}


	/*
	 * Handle dismiss, close, etc by just hiding ourselves.
	 */
	private void handle_ok_button() {

		// simply just hide this frame from view
		setVisible( false );
	}


	/*
	 * Handle GUI component actions and events.
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof Button ) {
			if ( evt.getSource() == ok_button ) {
				handle_ok_button();
			}
		}
	}
	public void windowClosing( WindowEvent evt ) {
		handle_ok_button();
	}
	public void windowClosed( WindowEvent evt ) {}
	public void windowOpened( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowIconified( WindowEvent evt ) {}
	public void windowDeiconified( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowActivated( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowDeactivated( WindowEvent evt ) {}


	/*
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );
		Frame frame = new Frame();
		frame.setSize( 100, 100 );
		frame.setVisible( true );
		AboutGUI about = new AboutGUI( false, frame, false,
				Locale.getDefault() );
		about.pack(); about.setVisible( true );
	}
}
