//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 1999  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// Finger class - some minimal information about fingers
//
import java.util.Vector;
import java.util.Enumeration;

/**
 * Finger class gives an enumerated typedef of fingers.
 */
public class Finger {

	// useful, and only, definitions of finger ids
	public static final Finger NO_FINGER	= new Finger( "NO_FINGER" );
	public static final Finger LH_THUMB	= new Finger( "LH_THUMB" );
	public static final Finger LH_INDEX	= new Finger( "LH_INDEX" );
	public static final Finger LH_MIDDLE	= new Finger( "LH_MIDDLE" );
	public static final Finger LH_RING	= new Finger( "LH_RING" );
	public static final Finger LH_PINKY	= new Finger( "LH_PINKY" );
	public static final Finger RH_THUMB	= new Finger( "RH_THUMB" );
	public static final Finger RH_INDEX	= new Finger( "RH_INDEX" );
	public static final Finger RH_MIDDLE	= new Finger( "RH_MIDDLE" );
	public static final Finger RH_RING	= new Finger( "RH_RING" );
	public static final Finger RH_PINKY	= new Finger( "RH_PINKY" );

	// class vector encompassing all fingers, and a static block
	// to populate it
	private static Vector   fingers  = new Vector();
	static {
		fingers.addElement( LH_PINKY ); fingers.addElement( LH_RING );
		fingers.addElement( LH_MIDDLE ); fingers.addElement( LH_INDEX );
		fingers.addElement( LH_THUMB ); fingers.addElement( RH_THUMB );
		fingers.addElement( RH_INDEX ); fingers.addElement( RH_MIDDLE );
		fingers.addElement( RH_RING ); fingers.addElement( RH_PINKY );
	}

	/**
	 * Class method to allow enumeration of fingers.
	 */
	public static Enumeration get_elements() {
		return ( fingers.elements() );
	}

	// instance value, constructor, resource id, left/right hand, and
	// toString methods
	private String id	= null;
	private Finger( String id ) {
		this.id		= id;
	}
	public String get_resource_id() {
		return ( "Finger." + id );
	}
	public boolean	is_left_handed(){
		return ( this == LH_PINKY || this == LH_RING
			|| this == LH_MIDDLE || this == LH_INDEX
			|| this == LH_THUMB );
	}
	public boolean	is_right_handed(){
		return ( this == RH_PINKY || this == RH_RING
			|| this == RH_MIDDLE || this == RH_INDEX
			|| this == RH_THUMB );
	}
	public String	toString()	{ return ( id ); }


	// test code
	public static void main ( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		for ( Enumeration e = Finger.get_elements();
				e.hasMoreElements(); )
			System.out.println( e.nextElement() );
	}
}
