//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 1999  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// GlyphGroup class - definitions of key glyph groupings
//
import java.util.Vector;
import java.util.Enumeration;

/**
 * GlyphGroup provides an enumerated typedef of key glyph groupings.
 */
public class GlyphGroup {

	// useful definitions key set grouping values
	public static final GlyphGroup NO_GROUP
					= new GlyphGroup( "NO_GROUP" );
	public static final GlyphGroup UNSHIFTED_ALPHA
					= new GlyphGroup( "UNSHIFTED_ALPHA" );
	public static final GlyphGroup SHIFTED_ALPHA
					= new GlyphGroup( "SHIFTED_ALPHA" );
	public static final GlyphGroup NUMERIC
					= new GlyphGroup( "NUMERIC" );
	public static final GlyphGroup FIGURES
					= new GlyphGroup( "FIGURES" );
	public static final GlyphGroup BASIC
					= new GlyphGroup( "BASIC" );

	// class vector encompassing all glyphgroups, and a static block
	// to populate it
	private static Vector	groups	= new Vector();
	static {
		groups.addElement( UNSHIFTED_ALPHA );
		groups.addElement( SHIFTED_ALPHA );
		groups.addElement( NUMERIC );
		groups.addElement( FIGURES );
		groups.addElement( BASIC );
	}

	// class methods to allow enumeration
	public static Enumeration get_elements() {
		return ( groups.elements() );
	}

	// instance value, constructor, resource id, and toString
	private String id	= null;
	private GlyphGroup( String id ) {
		this.id		= id;
	}
	public String get_resource_id() {
		return ( "GlyphGroup." + id );
	}
	public String	toString()	{ return ( id ); }


	/**
	 * Class test code.
	 */
	public static void main ( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		for ( Enumeration e = GlyphGroup.get_elements();
				e.hasMoreElements(); )
			System.out.println( e.nextElement() );
	}
}
