//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 1999  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// HelpGUI class - creates a frame to display some online help
//
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.ListResourceBundle;
import java.text.MessageFormat;

/**
 * Creates a frame to display some online help.
 */
public class HelpGUI extends Frame
		implements ItemListener, ActionListener, WindowListener {
	private boolean	inAnApplet	= true;
	private Locale	locale		= null;

	// message catalog information
	private ListResourceBundle	messages	= null;

	// the help text itself, lazy_initialized; this way, if the user
	// never reads help text then the bulk data class should never
	// get loaded in
	private String		HTML_help_text[]	= null;
	private String		unHTML_help_text[]	= null;

	// menubar GUI components
	private MenuBar		menu_bar;
	private Menu		file_menu;
	private MenuItem	file_next, file_prev, file_close;

	// GUI components
	private Choice		topic_choice;
	private TextArea	text_area;
	private Label		status_label;


	/**
	 * Initialize the whole GUI set.
	 */
	public HelpGUI( boolean inAnApplet, Locale locale ) {
		this.addWindowListener( this );
		this.inAnApplet	= inAnApplet;
		this.locale	= locale;

		// find the resource bundle for the localized messages
		messages = (ListResourceBundle)
				(ResourceBundle.getBundle(
					"MessageResource", locale ));

		// create GUI components
		setTitle( messages.getString( "HelpGUI.title" ));
		setLayout( new BorderLayout() );

		// create the menu bar stuff
		menu_bar = new MenuBar();
		setMenuBar( menu_bar );
		// File menu
		file_menu = new Menu( messages.getString( "HelpGUI.file" ));
		menu_bar.add( file_menu );
		file_prev = new MenuItem(
			messages.getString( "HelpGUI.file_prev" ),
			new MenuShortcut(
				((Integer)messages.getObject(
				"HelpGUI.file_prev_sc" )).intValue() ));
		file_prev.addActionListener( this );
		file_menu.add( file_prev );
		file_next = new MenuItem(
			messages.getString( "HelpGUI.file_next" ),
			new MenuShortcut(
				((Integer)messages.getObject(
				"HelpGUI.file_next_sc" )).intValue() ));
		file_next.addActionListener( this );
		file_menu.add( file_next );
		file_close = new MenuItem(
			messages.getString( "HelpGUI.file_close" ),
			new MenuShortcut(
				((Integer)messages.getObject(
				"HelpGUI.file_close_sc" )).intValue() ));
		file_close.addActionListener( this );
		file_menu.add( file_close );

		// create the text display
		text_area = new TextArea( 12, 80 );
		text_area.setBackground( Color.white );
		text_area.setEditable( false );
		add( "Center", text_area );

		status_label = new Label(
			messages.getString( "HelpGUI.nohelp" ), Label.RIGHT );
		add( "South", status_label );

		// add topic choice as a toolbar item
		Panel toolbar = new Panel();
		toolbar.setLayout( new BorderLayout() );
		add( "North", toolbar );

		Panel controls = new Panel();
		controls.setLayout( new FlowLayout() );
		toolbar.add( "West", controls );
		controls.add( new Label(
			messages.getString( "HelpGUI.topicchoice" )));
		topic_choice = new Choice();
		topic_choice.addItemListener( this );
		controls.add( topic_choice );

		// disable these menu items until initialization is done
		file_prev.setEnabled( false );
		file_next.setEnabled( false );

		// add a bogus entry to topic choice just to give
		// the thing dimensions; I don't know why this should
		// be necessary to do
		topic_choice.add(
			messages.getString( "HelpGUI.dummy" ));
	}


	/*
	 * Convenience functions to find topic_name and topic_text
	 * from a help topic string.
	 */
	private String get_topic_name( String s ) {
		return ( s.substring( 0, s.indexOf( "\n" )));
	}
	private String get_topic_text( String s ) {
		return ( s.substring( s.indexOf( "\n" ) + 1, s.length() ));
	}


	/*
	 * Take out the HTML elements from a help string - we don't do them
	 * here.
	 */
	private String unHTML( String s ) {
		StringBuffer buffer = new StringBuffer();

		// pass over the string removing HTML <> stuff and
		// translating &...; items (awk would be nice here...)
		boolean in_html_tag = false;
		for ( int index = 0; index < s.length(); index++ ) {
			if ( index < s.length() - 4 &&
				s.regionMatches( index, "&lt;", 0, 4 )) {
				buffer.append( "<" ); index += 3; continue;
			}
			else if ( index < s.length() - 4 &&
				s.regionMatches( index, "&gt;", 0, 4 )) {
				buffer.append( ">" ); index += 3; continue;
			}
			else if ( index < s.length() - 5 &&
				s.regionMatches( index, "&amp;", 0, 5 )) {
				buffer.append( "&" ); index += 4; continue;
			}
			else if ( index < s.length() - 6 &&
				s.regionMatches( index, "&quot;", 0, 6 )) {
				buffer.append( "\"") ; index += 5; continue;
			}
			else if ( index < s.length() - 6 &&
				s.regionMatches( index, "&#xxx;", 0, 2 ) &&
				s.regionMatches( index + 5, "&#xxx;", 5, 1 )) {
				index += 5; continue;
			}
			else {
				String c = s.substring( index, index + 1 );
				if ( c.equals( "<" )) {
					in_html_tag = true; continue;
				}
				else if ( c.equals( ">" )) {
					in_html_tag = false; continue;
				}
				else if ( ! in_html_tag )
					buffer.append( c );
			}
		}
		return ( buffer.toString() );
	}


	/*
	 * Instantiate the relevant help text class, and return the help
	 * text from it.
	 */
	private String[] get_help_text() {

		// find the help text resource for our locale
		status_label.setText( messages.getString( "HelpGUI.loading" ));
		ListResourceBundle resource = (ListResourceBundle)
				(ResourceBundle.getBundle(
					"HelpTextResource", locale ));
		status_label.setText( messages.getString( "HelpGUI.ready" ));

		// return the help text data, as a string array
		return ( resource.getStringArray( "helptext" ) );
	}


	/*
	 * Build the choice of help topics from the help text array.
	 */
	private void find_topics( String[] help, Choice choice ) {

		// build a choice of topics from each help text string
		for ( int index = 0; index < help.length; index++ )
			choice.addItem( get_topic_name( help[ index ] ));
	}


	/**
	 * Override setVisible with a method that completes our initialization
	 * of the help text; this way, if the frame is never rendered
	 * then the help text isn't downloaded.
	 */
	public void setVisible( boolean is_visible ) {
		if ( is_visible ) {

			// see if we have initialized the help text already
			// and if not, then get it
			if ( HTML_help_text == null ) {

				// get the help text
				HTML_help_text = get_help_text();

				// drop the bogus entry
				topic_choice.removeAll();

				// build a choice of topics
				find_topics( HTML_help_text, topic_choice );

				// initiate display of the first topic
				topic_choice.select( 0 );
				handle_topic_selection();
			}

			// call the original setVisible
			super.setVisible( true );
		}
		else
			super.setVisible( false );
	}


	/*
	 * Handle dismiss, close, etc by just hiding ourselves.
	 */
	private void handle_close_button() {

		// simply just hide this frame from view
		setVisible( false );
	}


	/*
	 * Handle the previous topic button.
	 */
	private void handle_prev_button() {

		// move to the prior help topic if there is one
		int selected = topic_choice.getSelectedIndex();
		if ( selected > 0 ) {
			selected--;
			topic_choice.select( selected );
			handle_topic_selection();
		}
	}


	/*
	 * Handle the next topic button.
	 */
	private void handle_next_button() {

		// move to the next help topic if there is one
		int selected = topic_choice.getSelectedIndex();
		if ( selected < topic_choice.getItemCount() - 1 ) {
			selected++;
			topic_choice.select( selected );
			handle_topic_selection();
		}
	}


	/*
	 * Handle the selection of a new series.
	 */
	private void handle_topic_selection() {

		// lazy initialization of the flat text version of help
		if ( unHTML_help_text == null )
			unHTML_help_text =
					new String[ HTML_help_text.length ];

		// lazy conversion of a help topic from HTML into flat text
		int index = topic_choice.getSelectedIndex();
		if ( unHTML_help_text[ index ] == null ) {

			// topic not yet converted, so convert it here
			String[] fmtargs = { get_topic_name(
					HTML_help_text[ index ] ) };
			status_label.setText(
				MessageFormat.format(
				messages.getString( "HelpGUI.formatting" ),
						fmtargs ));
			unHTML_help_text[ index ] =
					unHTML( HTML_help_text[ index ]);
			status_label.setText(
				messages.getString( "HelpGUI.ready" ));
		}

		// display the text for the selected topic
		text_area.setText( get_topic_text(
				unHTML_help_text[ index ]));


		// update the sensitivity of the next/prev menu items
		if ( index > 0 )
			file_prev.setEnabled( true );
		else
			file_prev.setEnabled( false );
		if ( index < topic_choice.getItemCount() - 1 )
			file_next.setEnabled( true );
		else
			file_next.setEnabled( false );
	}


	/*
	 * Handle GUI component actions and events.
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof MenuItem ) {
			if ( evt.getSource() == file_close )
				handle_close_button();
			else if ( evt.getSource() == file_prev )
				handle_prev_button();
			else if ( evt.getSource() == file_next )
				handle_next_button();
		}
	}
	public void itemStateChanged( ItemEvent evt ) {
		if ( evt.getSource() instanceof Choice ) {
			handle_topic_selection();
		}
	}
	public void windowClosing( WindowEvent evt ) {
		handle_close_button();
	}
	public void windowClosed( WindowEvent evt ) {}
	public void windowOpened( WindowEvent evt ) {
		topic_choice.requestFocus();
	}
	public void windowIconified( WindowEvent evt ) {}
	public void windowDeiconified( WindowEvent evt ) {
		topic_choice.requestFocus();
	}
	public void windowActivated( WindowEvent evt ) {
		topic_choice.requestFocus();
	}
	public void windowDeactivated( WindowEvent evt ) {}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );
		HelpGUI help = new HelpGUI( false, Locale.getDefault() );
		help.pack(); help.setVisible( true );
	}
}
