//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 1999  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// HelpTextResource class - the fallback (English) online documentation
//
import java.util.ListResourceBundle;

/**
 * Fallback (English) language online documentation text for Typist.
 * <p>
 * Anyone translating the program should convert this file into the
 * required language, and store as HelpTextResource_&lt;language&gt;java.
 * <p>
 * Text is stored in pidgin-HTML, and is used for both the online help
 * window and to make the HTML doc file for the package.
 * <p>
 * The whole file contains a single string array resource.  Thus the
 * ResourceBundle lookup facilities aren't used; just the locale processing.
 * The single resource is "helptext".
 */
public class HelpTextResource extends ListResourceBundle {

	// handle on the contents of this catalog
	public Object[][] getContents() { return contents; }

// BEGIN LOCALIZATION SECTION

	static final String[] help_data = {
"Introduction\n"
+ "<B>Typist</B> is a program to provide you with instruction and practice in the\n"
+ "use of a computer keyboard.\n"
+ "<P>\n"
+ "If you cannot touch type, <B>Typist</B> contains all the tutorials and practice\n"
+ "text you will need in order to master this skill.  And if you can already touch\n"
+ "type, then you can use the program to sharpen your keyboard skills, and improve\n"
+ "your typing speed and efficiency.\n"
+ "<P>\n"
+ "<B>Typist</B> contains lessons and practice text for the standard QWERTY keyboard,\n"
+ "the Dvorak (simplified) keyboard, and the PC numeric keypad.  On each typing\n"
+ "exercise, it measures your speed, and can provide you with information about\n"
+ "which keys or fingers need additional practice.\n",

"Using Online Help\n"
+ "You can use the <I>Help Window</I> to display online help while running\n"
+ "<B>Typist</B>.  At the top of the window you can select from the topics that\n"
+ "the help window offers.  Selecting a topic brings the help information\n"
+ "for that area into the window's text display.\n"
+ "<P>\n"
+ "To reduce program startup time, the online help text is not accessed until\n"
+ "the <I>Help Window</I> is first displayed.  This is particularly useful when\n"
+ "<B>Typist</B> is running as an applet.  You may however see a delay when\n"
+ "accessing the online help for the first time.\n"
+ "<P>\n"
+ "Select <I>File-&gt;Close Window</I> on the menu bar for this window to close\n"
+ "it.\n"
+ "<P>\n"
+ "The file <I>Typist31.html</I> contains an HTML version of the online help\n"
+ "text.\n",

"The Control Window\n"
+ "This is the main window for <B>Typist</B>.  The menu bar allows you to\n"
+ "activate the other windows that <B>Typist</B> can display -- the <A HREF=\"#topic_3\"><I>Tutorial\n"
+ "Window</I></A>, the <A HREF=\"#topic_4\"><I>Exercise Window</I></A>, the <A HREF=\"#topic_5\"><I>Keyboard Window</I></A>, the <A HREF=\"#topic_6\"><I>Statistics\n"
+ "Window</I></A>, and the online <A HREF=\"#topic_1\"><I>Help Window</I></A>.  Additionally, it also offers a <I>Help</I>\n"
+ "pulldown item: Use <I>Help-&gt;About</I> to get information about <B>Typist</B> and\n"
+ "its licensing, and <I>Help-&gt;On Typist</I> to obtain the online <A HREF=\"#topic_1\"><I>Help Window</I></A>.\n"
+ "<P>\n"
+ "<B>Typist</B> offers a number of alternative and complementary lesson series.\n"
+ "The <I>Control Window</I> allows you to select which lesson series is\n"
+ "currently active, and within a lesson series allows you to select a\n"
+ "particular lesson.\n"
+ "<P>\n"
+ "Beneath the window's menu bar is a toolbar.  Using the toolbar, you can\n"
+ "select the lesson series that is active, and move forwards and backwards\n"
+ "through the lessons in that series using the <I>&lt;Prev</I> and <I>Next&gt;</I>\n"
+ "buttons.\n"
+ "<P>\n"
+ "The central area of the <I>Control Window</I> is the lessons list.  This shows\n"
+ "each lesson that the currently active series contains.  To run any lesson\n"
+ "individually, select it from this list.  Lessons can be of three basic\n"
+ "types:\n"
+ "<P>\n"
+ "    <UL><LI><I>Tutorials</I></LI>\n"
+ "    <LI><I>Drills</I></LI>\n"
+ "    <LI><I>Speed Tests</I></LI></UL>\n"
+ "<P>\n"
+ "<I>Tutorials</I> are displayed in the <A HREF=\"#topic_3\"><I>Tutorial Window</I></A>.  <I>Drills</I> and <I>Speed Tests</I>\n"
+ "are displayed in the <A HREF=\"#topic_4\"><I>Exercise Window</I></A>.\n",
 
"The Tutorial Window\n"
+ "This window shows the text of the most recently selected <I>Tutorial</I>.\n"
+ "Use the scroll bars to read the text.  The window may be left up as a\n"
+ "reminder of the lesson if you wish.\n"
+ "<P>\n"
+ "This window also maintains a list of several past tutorials you have viewed.\n"
+ "Use the <I>History</I> menu on the menu bar to review them.\n",

"The Exercise Window\n"
+ "This is the window that you type into for practice, and displays both\n"
+ "<I>Drills</I> and <I>Speed Tests.</I>\n"
+ "<P>\n"
+ "When a <I>Drill</I> is being shown, the window contains two text areas.  The\n"
+ "upper one shows the target text that should be typed, highlighting the\n"
+ "next character expected.  The lower text area shows what you have\n"
+ "typed at the keyboard so far, with error characters picked out as\n"
+ "'*'s.  In a <I>Drill</I>, backspace characters will not delete the last typed\n"
+ "character.\n"
+ "<P>\n"
+ "For a <I>Speed Test</I>, only one text area is shown.  Again, the expected next\n"
+ "character is highlighted.  When in a <I>Speed Test</I>, you need to overtype\n"
+ "the characters in this text area.  If you enter an incorrect character,\n"
+ "<B>Typist</B> puts a '*' at the location of the error.  You can use backspace\n"
+ "to back up and correct mis-typings in <I>Speed Tests</I>.\n"
+ "<P>\n"
+ "For both of these types of exercise, <B>Typist</B> times each keystroke, and\n"
+ "at the end of the exercise, it displays information about how many\n"
+ "errors you made in typing the exercise, and also your typing speed and\n"
+ "adjusted speed in Words Per Minute (WPM).  Your raw typing speed is\n"
+ "calculated as the number of keystrokes you made, divided by five (the\n"
+ "notional average word is taken as being five characters), divided by\n"
+ "the time in minutes taken to complete the exercise.  The adjusted WPM\n"
+ "is calculated in the same way, except that one word is taken away for\n"
+ "each single character typing error you made.\n"
+ "<P>\n"
+ "When timing keystrokes, <B>Typist</B> does not count the first character\n"
+ "of an exercise.  Also, if you pause for a few seconds between characters\n"
+ "it won't count the next character after the pause as part of the exercise.\n"
+ "It does this to try to maintain sensible keystroke timing information in\n"
+ "cases where you might take breaks in mid-exercise.\n"
+ "<P>\n"
+ "By default, <B>Typist</B> will try to behave like a word processor.  A space\n"
+ "character is treated as correctly typed at the end of a line, and multiple\n"
+ "spaces are condensed down into a single space.  If you hit Return at the end\n"
+ "of a line, the cursor moves forward to the next significant character to be\n"
+ "typed, skipping over blank lines and paragraph indentation.  <B>Typist</B> won't\n"
+ "insist that you type the hyphen at the end of lines that end in one, but will\n"
+ "wrap for you.  To switch off any or all of these word processing features,\n"
+ "use the <I>Options</I> menu on the menu bar.\n"
+ "<P>\n"
+ "To restart an exercise from the beginning, use <I>File-&gt;Restart Lesson</I>\n"
+ "on the menu bar.\n"
+ "<P>\n"
+ "Like the <A HREF=\"#topic_3\"><I>Tutorial Window</I></A>, this window maintains a list of several past\n"
+ "exercises you have worked on.  Use the <I>History</I> menu on the menu bar to\n"
+ "re-run these exercises.\n",

"The Keyboard Window\n"
+ "The <I>Keyboard Window</I> shows a picture of your keyboard.  When <B>Typist</B> is\n"
+ "expecting you to type a key in an exercise, it indicates the key that it is\n"
+ "waiting for in this window.  It also indicates the finger that you should use\n"
+ "to strike that key, and which <I>Shift</I> key you should press, if any.\n"
+ "<P>\n"
+ "The <I>Keyboard Window</I> can also show you your successful keystrokes, and any\n"
+ "mistakes that you may make in an exercise.  Use the <I>Options</I> menu on the menu\n"
+ "bar to select what things you want the <I>Keyboard Window</I> to show you.\n"
+ "<P>\n"
+ "Note that only fingering for the main keyboard is indicated by this window.\n"
+ "It does not show you which finger you should use for the numeric keypad.\n"
+ "Also, the numeric keypad is not controlled by the keyboard layout you are\n"
+ "using; <B>Typist</B> assumes it is the same for every keyboard type.\n"
+ "<P>\n"
+ "Although it is a useful learning aid, especially in the early stages of\n"
+ "learning the keyboard, you should aim to practice without the <I>Keyboard\n"
+ "Window</I> as soon as you can.\n",

"The Statistics Window\n"
+ "Each time you strike a key in an exercise, <B>Typist</B> notes the key\n"
+ "that you struck, the one it was expecting you to strike, and the time\n"
+ "it took you between the last key you struck and this one.\n"
+ "<P>\n"
+ "The <I>Statistics Window</I> displays a complete analysis of all of\n"
+ "this timing information collected on exercises.  Keystroke statistics are\n"
+ "integrated globally, by finger, and displayed by individual key.  You can\n"
+ "use the <I>View</I> pulldown menu to decide which set of statistics you\n"
+ "want to see in the window.\n"
+ "<P>\n"
+ "Each statistic is shown as a pair of gauges, giving the percent accuracy\n"
+ "and speed for the finger or key.  If the accuracy or speed drop below\n"
+ "certain threshold levels, the gauges will change color to orange or red,\n"
+ "to indicate that these are areas in which you might consider some additional\n"
+ "practice.\n"
+ "<P>\n"
+ "Press the <I>Refresh</I> button to bring the window up to date with current\n"
+ "keystroke statistics.  The window is automatically refreshed if you change\n"
+ "which set of statistics is on view, when the window is displayed, and at\n"
+ "the end of each typing exercise you complete.  You can also use the autorefesh\n"
+ "function to have the statistics refresh automatically.  To control this\n"
+ "feature, use the <I>Options-&gt;Automatic Refresh Every Second</I> pulldown\n"
+ "menu item.\n"
+ "<P>\n"
+ "You can use the <I>File-&gt;Reset All</I> pulldown menu item to set the\n"
+ "complete collection of measurements back to their initial condition.\n"
+ "To display adjusted WPM statistics, use the <I>Options-&gt;Display Adjusted\n"
+ "WPM</I> pulldown menu item.\n"
+ "<P>\n"
+ "The statistics shown by this window are the normal keystroke statistics,\n"
+ "broken down by key and by finger.  Note that the WPM rates are calculated\n"
+ "on timing information that is maintained regardless of whether you struck\n"
+ "the right key or the wrong one -- this means that where accuracy is poor,\n"
+ "the WPM rates shown for individual keys and fingers can become misleading.",

"The Lesson Loader\n"
+ "Because a large amount of <B>Typist</B> is exercises and other typing practise\n"
+ "material, it makes sense for the program to only load those lesson series that\n"
+ "are actually required.  It does this by using dynamic class file loading to\n"
+ "access a lesson series when it is selected on the <A HREF=\"#topic_2\"><I>Control Window</I></A>.  Once\n"
+ "a lesson series has been loaded, <B>Typist</B> keeps it in memory, so there is\n"
+ "no need to load it again.\n"
+ "<P>\n"
+ "The effect of this is that the startup time is reduced where <B>Typist</B> is\n"
+ "being run as an applet.  You may however notice a delay due to downloading\n"
+ "when you access a lesson series for the first time.  Dynamic lesson loading\n"
+ "happens automatically, and does not require you to take any special actions.\n",

"Building The Program\n"
+ "<B>Typist</B> (Java version) consists of a flat directory of Java source\n"
+ "files, and a single <I>Makefile</I>.  If you have <I>make</I> on your\n"
+ "system, then merely invoking the command should be sufficient to build\n"
+ "the complete set of class files -- the <I>Makefile</I> does not contain\n"
+ "any complicated or GNU-make dependent constructs.\n"
+ "<P>\n"
+ "If you do not have <I>make</I> available, try '<I>javac [-O] *.java</I>'.\n"
+ "The Java compiler is sometimes smart enough to automatically compile any\n"
+ "source files for a needed class file, so building may not be too much of\n"
+ "a problem.\n"
+ "<P>\n"
+ "It is not necessary to install the program in order to run it.  If you do\n"
+ "choose to install the program, however, as an application, the command to\n"
+ "do this is '<I>make install_application</I>'.  Installation of the program as an\n"
+ "applet depends on the Web server you are using, and is not automated by the\n"
+ "Makefile.  Typically, all you should need to do is to copy the class files and\n"
+ "the HTML files into a subdirectory of your Web server's main documents\n"
+ "directory (for example, <I>htdocs</I> on Apache).  Please consult your Web server's\n"
+ "documentation if you are unsure of how to do this.\n"
+ "<P>\n"
+ "Most classes contain a <I>main()</I> method that tests the class in some way, the\n"
+ "obvious exception being the Typist class that contains the 'real' <I>main()</I>\n"
+ "method, and also the applet entry point for the program.  The <I>main()</I>\n"
+ "method in the HelpHTML class prints out the online help text as a single\n"
+ "HTML User Guide document.\n",

"Running The Program\n"
+ "The single main runnable class file in <B>Typist</B> is:\n"
+ "<P>\n"
+ "    <UL><LI>Typist.class</LI></UL>\n"
+ "<P>\n"
+ "To run <B>Typist</B> as an application; use a command something like '<I>java\n"
+ "Typist</I>' to run the <I>Typist</I> class.  You can add <I>-k keyboard_type</I>\n"
+ "to specify the keyboard type.  Use <I>-h</I> for a full list of arguments.\n"
+ "<P>\n"
+ "If you have installed the program using '<I>make install_application</I>', you\n"
+ "can run the program using the simple command '<I>jtypist</I>', found in the\n"
+ "directory <I>/usr/local/bin</I>.\n"
+ "<P>\n"
+ "<B>Typist</B> can translate between keyboard types for you as it runs.  To do this\n"
+ "you need to add the option <I>-r real_keyboard_type</I>.  This can be confusing,\n"
+ "so here is an example.  Say you are using a US keyboard, and want to try out\n"
+ "<B>Typist</B> in Dvorak mode; you can do this by adding the options '<I>-k Dvorak\n"
+ "-r US</I>'.  This tells <B>Typist</B> to map each keystroke from your US keyboard\n"
+ "into its Dvorak equivalent.  You should note, however, that <B>Typist</B> does not\n"
+ "translate menu accelerator keys; in all cases, these need to be entered using\n"
+ "the keys shown on the actual system keyboard.\n"
+ "<P>\n"
+ "If you have localized messages and help text, <I>Typist</I> will use them,\n"
+ "obtaining the locale from the Java runtime's default locale setting.  You can\n"
+ "also request a specific locale using the <I>-l</I> or <I>-locale</I> option.\n"
+ "See <A HREF=\"#topic_12\"><I>Localization</I></A> for details on how to specify a locale."
+ "<P>\n"
+ "To run <B>Typist</B> as an applet, again use the <I>Typist</I> class.  For an\n"
+ "an applet, the keyboard type can be specified using the applet parameter\n"
+ "<I>kbtype</I> -- see <I>TypistApplet.html</I> for an example of HTML applet tags\n"
+ "to activate the program.  Because the program makes heavy use of frames and\n"
+ "menu bars, the applet form is nothing more than a single pushbutton that\n"
+ "starts the main program in a separate frame outside the browser.\n"
+ "<P>\n"
+ "Applet parameters include:\n"
+ "<P>\n"
+ "    <UL><LI>locale - the locale for messages and help text</LI>\n"
+ "    <LI>kbytpe - the keyboard type</LI>\n"
+ "    <LI>realkbtype - the real keyboard type, for translations</LI>\n"
+ "    <LI>series - a comma-separated list of the series to offer</LI>\n"
+ "    <LI>messages - whether to show status messages on startup</LI>\n"
+ "    <LI>buttonlabel - text to put on the applet's single pushbutton</LI></UL>\n"
+ "<P>\n"
+ "Note that it is not necessary to use a Web server in order to run <B>Typist</B>\n"
+ "as an applet.  You can, for example, simply point a JVM-capable browser at\n"
+ "the <I>TypistApplet.html</I> file using a '<I>file:/</I>' URL.\n",

"Adding New Lessons\n"
+ "<B>Typist</B> allows you to add new lessons to the ones that it offers.  To\n"
+ "do this, you need to create a lesson series class file.  For example, if you\n"
+ "want to create a lesson series <I>Foo</I>, the class file containing your lesson\n"
+ "should be named <I>LessonSeriesFoo.class</I>.  You can then use the <I>-s</I>\n"
+ "or <I>-series</I> flags, or the <I>series</I> applet parameter, to specify the series\n"
+ "that you want <B>Typist</B> to offer.  This is a comma-separated list, for\n"
+ "example, 'T,R,Q,Foo'.\n"
+ "<P>\n"
+ "To build the <I>LessonSeriesFoo.class</I> file, copy an existing lesson series java\n"
+ "source file to <I>LessonSeriesFoo.java</I>, and edit the file to contain the lessons\n"
+ "you want to put in this series; use the existing files as an example of format.\n"
+ "When finished, compile the java source file.  Javac, guavac, or any other Java\n"
+ "compiler should be suitable.\n"
+ "<P>\n"
+ "Each lesson in a series needs to be one of the three allowable lesson types.\n"
+ "For tutorials, a lesson title needs to be given, along with the tutorial text.\n"
+ "For exercises, both a title and a brief set of instructions may accompany the\n"
+ "exercise text.\n",

"Adding New Keyboards\n"
+ "<B>Typist</B> also allows you to extend its range of keyboard maps.  To do this,\n"
+ "you need to create a new <I>KBLayout</I> class file.  You can then use the\n"
+ "<I>-k</I> or <I>-kb</I> flags, or the <I>kbtype</I> applet parameter, to tell the\n"
+ "program to use the new keyboard map.  The new map can also be used with the\n"
+ "<I>-r</I> or <I>-realkb</I> flags, or the <I>realkbtype</I> applet parameter, too.\n"
+ "<P>\n"
+ "As with adding a new lesson series, the simplest way to create the <I>KBLayout</I>\n"
+ "class file is to copy an existing keyboard layout, and use it as a template.\n"
+ "For example, to create <I>KBLayoutFoo.class</I>, copy <I>KBLayoutUS.java</I> to the file\n"
+ "<I>KBLayoutFoo.java</I>, and edit this file to contain your layout.  When finished,\n"
+ "compile this file to create <I>KBLayoutFoo.class</I>, and then run <B>Typist</B> with\n"
+ "the option <I>-k Foo</I> to use the new layout.\n"
+ "<P>\n"
+ "For each keyboard character, the new layout needs to specify the finger that\n"
+ "is assigned to the key, whether the key is shifted, the group the character\n"
+ "is in for statistical purposes, and the keycode of the key that creates the\n"
+ "character.  <B>Typist</B> will create the <A HREF=\"#topic_5\"><I>Keyboard Window</I></A> by using the\n"
+ "following table to turn keycodes into key positions in the window:\n"
+ "<P>\n<PRE>"
+ "    Row 1: 41, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12, 13,&#032;&#032; 14\n"
+ "    Row 2:&#032; 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27,&#032; 43\n"
+ "    Row 3:&#032; 58,&#032; 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,&#032;&#032;&#032;&#032; 28\n"
+ "    Row 4:&#032;&#032; 42,&#032; 44, 45, 46, 47, 48, 49, 50, 51, 52, 53,&#032;&#032;&#032;&#032;&#032;&#032;&#032; 54\n"
+ "    Row 5:&#032;"
+ " 29,&#032;&#032;&#032;"
+ " 56,&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;"
+ " 57,&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;"
+ " 100,&#032;&#032;&#032;&#032;&#032;&#032;&#032;&#032;"
+ " 97\n"
+ "</PRE>",

"Localization\n"
+ "<B>Typist</B> is written to allow localization of messages, GUI elements, and\n"
+ "online help text.  There are two resources that you should translate if you\n"
+ "intend to create localized strings for the program:\n"
+ "<P>\n"
+ "    <UL><LI><I>MessageResource.java</I></LI>\n"
+ "    <LI><I>HelpTextResource.java</I></LI></UL>\n"
+ "<P>\n"
+ "The first of these contains the messages, GUI element labels, and GUI menu\n"
+ "shortcuts - in short, all program text except the help text.  To create a\n"
+ "national language version of this class, copy the file to the new file\n"
+ "<I>MessageResource_&lt;nn&gt;.java</I>, where &lt;nn&gt; is the ISO code for the\n"
+ "language you are translating for, edit this file and translate each text\n"
+ "message in it, and compile it.\n"
+ "<P>\n"
+ "The second file is the help text for the program, also used to generate the\n"
+ "HTML User Guide document.  Translation for this file is the same as for the\n"
+ "messages.  The reason <I>Typist</I> splits internationalization into two files is\n"
+ "so that loading the help text may be avoided in cases where the user does not\n"
+ "access it in any way.\n"
+ "<P>\n"
+ "To run <I>Typist</I> in a particular locale, use the <I>-l</I> or <I>-locale</I> flags,\n"
+ "or the <I>locale</I> applet parameter.  Locales are specified in the format\n"
+ "<I>nn</I>[_<I>CC</I>[_<I>variant</I>], where <I>nn</I> is the ISO language code (for example, \"en\", <I>CC</I> is\n"
+ "the ISO country code (for example, \"US\"), and <I>variant</I> is any local\n"
+ "variant of the locale.  Only the language part is required for a locale; the\n"
+ "country and variant may be omitted.  <I>Typist</I> will also pick up the default\n"
+ "locale set for the user, using it when no locale is specified to the program.\n"
+ "<P>\n"
+ "If there are no messages or help text for a particular locale, <I>Typist</I> will\n"
+ "fall back to English messages and help text.\n",

"Java Runtime Notes\n"
+ "<B>Typist</B> was developed on a combination of the 1.0.2 JDK and the 1.1.7\n"
+ "JDK. both running on Linux, but is now fully converted to use the Java 1.1\n"
+ "event model.  The following notes apply to the the various Java runtimes\n"
+ "tried so far:\n"
+ "<P><DL>\n"
+ "<DT><I>JDK 1.0.2 on Linux</I></DT><DD>\n"
+ "    This JVM is not AWT 1.1 based, and is no longer supported.\n"
+ "</DD>\n"
+ "<DT><I>JDK 1.1.7 on Linux</I></DT><DD>\n"
+ "    There are two annoying features that simply won't work with this runtime.\n"
+ "    The first is the Caps Lock key -- no matter what state keyboard Caps Lock\n"
+ "    is in, key press events passed in to the program never show uppercase\n"
+ "    letters; they are always lowercase.  The second is that numeric keypad\n"
+ "    key presses do not, apparently, generate key press events at all,\n"
+ "    rendering the numeric keypad useless.  There is a reasonable workround\n"
+ "    in place to handle Caps Lock in the program itself, but there is no\n"
+ "    workround possible for the numeric keypad problem, so this feature simply\n"
+ "    doesn't work.  Hopefully the situation will improve with later releases.\n"
+ "    The remainder of the program functions correctly.\n"
+ "</DD>\n"
+ "<DT><I>JDK 1.1.7B on SCO UnixWare 7</I></DT><DD>\n"
+ "    This behaves as JDK 1.1.7 above.  Neither Caps Lock nor the numeric\n"
+ "    keypad will function correctly on this runtime.  Also, I noticed some\n"
+ "    odd threads problems, that seemed to disappear when the program was\n"
+ "    recompiled on this platform.  The remainder of the program functions\n"
+ "    correctly.\n"
+ "</DD>\n"
+ "<DT><I>Kaffe 1.0b on Linux</I></DT><DD>\n"
+ "    Sadly, the program won't run in Kaffe 1.0b.  The GUI appears, but for\n"
+ "    some reason it receives no key event input at all.  A small amount of\n"
+ "    fiddling about with the GUI buttons and lists usually results in a core\n"
+ "    dump from Kaffe, sometimes accompanied by an assertion failure message.\n"
+ "</DD>\n"
+ "<DT><I>Netscape Navigator Gold 3.04 on UnixWare</I></DT><DD>\n"
+ "    The JVM in this browser is not AWT 1.1 based, and is no longer supported.\n"
+ "</DD>\n"
+ "<DT><I>Netscape Communicator 4.07 on Linux</I></DT><DD>\n"
+ "    Neither Caps Lock nor the numeric keypad work.  And a special treat -- on\n"
+ "    closing the applet, the browser will complain about finding circular\n"
+ "    dependencies in forms children, or generally give some other Motif error\n"
+ "    messages.  If you can live with this, however, the rest of the program\n"
+ "    behaves itself.\n"
+ "</DD>\n"
+ "<DT><I>Netscape Communicator 4.08 on UnixWare</I></DT><DD>\n"
+ "    Caps Lock works fine.  In other respects, however, this has the same\n"
+ "    problems as Communicator 4.07 for Linux.\n"
+ "</DD>\n"
+ "<DT><I>Netscape Communicator 4.5 on Windows</I></DT><DD>\n"
+ "    At last, a browser-based runtime that seems, at least so far, to run the\n"
+ "    complete applet without any of the mishaps described above.\n"
+ "</DD>\n"
+ "<DT><I>Netscape Communicator 4.61 on Windows</I></DT><DD>\n"
+ "    Again, no obvious problems.  Caps Lock works fine, and the program's\n"
+ "    workround, if activated when unnecessary, causes no ill effects.  It's\n"
+ "    unknown if the numeric keypad works, though; this test was performed on\n"
+ "    a laptop that had no numeric keypad.\n"
+ "</DD>\n"
+ "<DT><I>Netscape Communicator 4.61 on UnixWare</I></DT><DD>\n"
+ "    Apparently, Caps Lock has stopped working again, and the numeric keypad\n"
+ "    problem remains, as does the circular children dependency in forms children\n"
+ "    complaint.  By and large, however, the program works.\n"
+ "</DD></DL><P>\n"
+ "You may need to be prepared for a voyage of discovery with your Java\n"
+ "runtime.  At the very minimum, if you see any odd failures or other strange\n"
+ "behaviour with the program on a particular platform, and you haven't done so\n"
+ "already, try recompiling -- the Linux JDK used to build the distribution is\n"
+ "probably not bug-free either, so this may be all it takes.\n",

"Bugs and Omissions\n"
+ "The <I>Caps Lock</I> key does not work correctly on some AWT 1.1 implementations.\n"
+ "See <A HREF=\"#topic_13\"><I>Java Runtime Notes</I></A> for at least some details on which ones have\n"
+ "specific problems.  There is a workround in the program for this, selectable\n"
+ "from the <I>Options</I> menu of the <A HREF=\"#topic_4\"><I>Exercise Window</I></A>.\n"
+ "<P>\n"
+ "The numeric keypad does not generate KeyEvents on some AWT 1.1 implementations.\n"
+ "Again, see <A HREF=\"#topic_13\"><I>Java Runtime Notes</I></A> for details of which JVMs are affected.\n"
+ "No workround exists for this problem.\n"
+ "<P>\n"
+ "The exercise GUI interface is not as slick as it could be.  In particular the\n"
+ "slavish use by <B>Typist</B> of TextArea components does not give as much control\n"
+ "over things like text placement as might be desirable.\n"
+ "<P>\n"
+ "The dynamic loading of lesson series class files should be OK, but it is a\n"
+ "a relatively new feature, so hasn't had as much testing as the rest of the\n"
+ "program.  The same goes for loading keyboard layout files.\n"
+ "<P>\n"
+ "The gauge in the startup dialog popup never appears when the program is\n"
+ "started as an applet.  I have no idea why.\n",

"General Information\n"
+ "This program is a Java reworking of a UNIX C++ reworking of a UNIX\n"
+ "C/Curses reworking of an IBM PC reworking of an original DECUS VAX BASIC\n"
+ "program.  This Java version, <B>Typist</B> <I>v3.1</I> was created by Simon Baldwin,\n"
+ "<A HREF=MAILTO:simonb@sco.com>simonb@sco.com</A>.\n"
+ "<P>\n"
+ "Thanks to the original authors, whoever they were, to D. Jason Penney for\n"
+ "the initial conversion into C/Curses, and to George Ferguson for posting\n"
+ "it to the Internet.\n",

"GNU GPL\n"
+ "<I>Typist v3.1d (Java), Copyright (C) 1999  Simon Baldwin</I>\n"
+ "<P>\n"
+ "This program comes with <B>ABSOLUTELY NO WARRANTY</B>; for details\n"
+ "please see the file 'COPYING' supplied with the source code.\n"
+ "This is free software, and you are welcome to redistribute it\n"
+ "under certain conditions; again, see 'COPYING' for details.\n"
+ "This program is released under the GNU General Public License.\n",
	};

// END LOCALIZATION SECTION

	// wrapper as an object array for the ResourceBundle
	static final Object[][] contents = { { "helptext", help_data } };
}
