//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 1999  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// KBLayoutDvorak class - describes the PC/Mac Dvorak keyboard and its use
//

public class KBLayoutDvorak extends KBLayout
			implements KBNamespace {

	// unique ID for the keyboard type
	public static final String	KB_ID	= "Dvorak";

	public KBLayoutDvorak() { set_kb_id( KB_ID );
//
// This table defines the character, the keycode of the key that implements
// that character, whether shift should be pressed with this keycode to get
// this character, what finger is assigned to the key, and the group the
// character belongs to.
//
//                  Char   Code    Shifted?    Finger      Glyphgroup
//                  ----   ----    --------    ------      ----------
append(  new Glyph( '[',   KC_12,  UNSHIFTED,  RH_PINKY,   BASIC           ));
append(  new Glyph( ']',   KC_13,  UNSHIFTED,  RH_PINKY,   BASIC           ));
append(  new Glyph( '\\',  KC_43,  UNSHIFTED,  RH_PINKY,   BASIC           ));
append(  new Glyph( '\'',  KC_16,  UNSHIFTED,  LH_PINKY,   BASIC           ));
append(  new Glyph( ',',   KC_17,  UNSHIFTED,  LH_RING,    BASIC           ));
append(  new Glyph( '.',   KC_18,  UNSHIFTED,  LH_MIDDLE,  BASIC           ));
append(  new Glyph( '/',   KC_26,  UNSHIFTED,  RH_PINKY,   BASIC           ));
append(  new Glyph( '=',   KC_27,  UNSHIFTED,  RH_PINKY,   BASIC           ));
append(  new Glyph( '-',   KC_40,  UNSHIFTED,  RH_PINKY,   BASIC           ));
append(  new Glyph( ';',   KC_44,  UNSHIFTED,  LH_PINKY,   BASIC           ));
append(  new Glyph( '\n',  KC_28,  UNSHIFTED,  RH_PINKY,   BASIC           ));
append(  new Glyph( ' ',   KC_57,  UNSHIFTED,  RH_THUMB,   BASIC           ));

append(  new Glyph( '`',   KC_41,  UNSHIFTED,  LH_PINKY,   FIGURES         ));
append(  new Glyph( '~',   KC_41,  SHIFTED,    LH_PINKY,   FIGURES         ));
append(  new Glyph( '!',   KC_2,   SHIFTED,    LH_PINKY,   FIGURES         ));
append(  new Glyph( '@',   KC_3,   SHIFTED,    LH_RING,    FIGURES         ));
append(  new Glyph( '#',   KC_4,   SHIFTED,    LH_MIDDLE,  FIGURES         ));
append(  new Glyph( '$',   KC_5,   SHIFTED,    LH_INDEX,   FIGURES         ));
append(  new Glyph( '%',   KC_6,   SHIFTED,    LH_INDEX,   FIGURES         ));
append(  new Glyph( '^',   KC_7,   SHIFTED,    RH_INDEX,   FIGURES         ));
append(  new Glyph( '&',   KC_8,   SHIFTED,    RH_INDEX,   FIGURES         ));
append(  new Glyph( '*',   KC_9,   SHIFTED,    RH_MIDDLE,  FIGURES         ));
append(  new Glyph( '(',   KC_10,  SHIFTED,    RH_RING,    FIGURES         ));
append(  new Glyph( ')',   KC_11,  SHIFTED,    RH_PINKY,   FIGURES         ));
append(  new Glyph( '{',   KC_12,  SHIFTED,    RH_PINKY,   FIGURES         ));
append(  new Glyph( '}',   KC_13,  SHIFTED,    RH_PINKY,   FIGURES         ));

append(  new Glyph( '|',   KC_43,  SHIFTED,    RH_PINKY,   FIGURES         ));
append(  new Glyph( '"',   KC_16,  SHIFTED,    LH_PINKY,   FIGURES         ));
append(  new Glyph( '<',   KC_17,  SHIFTED,    LH_RING,    FIGURES         ));
append(  new Glyph( '>',   KC_18,  SHIFTED,    LH_MIDDLE,  FIGURES         ));
append(  new Glyph( '?',   KC_26,  SHIFTED,    RH_PINKY,   FIGURES         ));
append(  new Glyph( '+',   KC_27,  SHIFTED,    RH_PINKY,   FIGURES         ));
append(  new Glyph( '_',   KC_40,  SHIFTED,    RH_PINKY,   FIGURES         ));
append(  new Glyph( ':',   KC_44,  SHIFTED,    LH_PINKY,   FIGURES         ));

append(  new Glyph( '1',   KC_2,   UNSHIFTED,  LH_PINKY,   NUMERIC         ));
append(  new Glyph( '2',   KC_3,   UNSHIFTED,  LH_RING,    NUMERIC         ));
append(  new Glyph( '3',   KC_4,   UNSHIFTED,  LH_MIDDLE,  NUMERIC         ));
append(  new Glyph( '4',   KC_5,   UNSHIFTED,  LH_INDEX,   NUMERIC         ));
append(  new Glyph( '5',   KC_6,   UNSHIFTED,  LH_INDEX,   NUMERIC         ));
append(  new Glyph( '6',   KC_7,   UNSHIFTED,  RH_INDEX,   NUMERIC         ));
append(  new Glyph( '7',   KC_8,   UNSHIFTED,  RH_INDEX,   NUMERIC         ));
append(  new Glyph( '8',   KC_9,   UNSHIFTED,  RH_MIDDLE,  NUMERIC         ));
append(  new Glyph( '9',   KC_10,  UNSHIFTED,  RH_RING,    NUMERIC         ));
append(  new Glyph( '0',   KC_11,  UNSHIFTED,  RH_PINKY,   NUMERIC         ));

append(  new Glyph( 'P',   KC_19,  SHIFTED,    LH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'Y',   KC_20,  SHIFTED,    LH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'F',   KC_21,  SHIFTED,    RH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'G',   KC_22,  SHIFTED,    RH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'C',   KC_23,  SHIFTED,    RH_MIDDLE,  SHIFTED_ALPHA   ));
append(  new Glyph( 'R',   KC_24,  SHIFTED,    RH_RING,    SHIFTED_ALPHA   ));
append(  new Glyph( 'L',   KC_25,  SHIFTED,    RH_PINKY,   SHIFTED_ALPHA   ));
append(  new Glyph( 'A',   KC_30,  SHIFTED,    LH_PINKY,   SHIFTED_ALPHA   ));
append(  new Glyph( 'O',   KC_31,  SHIFTED,    LH_RING,    SHIFTED_ALPHA   ));
append(  new Glyph( 'E',   KC_32,  SHIFTED,    LH_MIDDLE,  SHIFTED_ALPHA   ));
append(  new Glyph( 'U',   KC_33,  SHIFTED,    LH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'I',   KC_34,  SHIFTED,    LH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'D',   KC_35,  SHIFTED,    RH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'H',   KC_36,  SHIFTED,    RH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'T',   KC_37,  SHIFTED,    RH_MIDDLE,  SHIFTED_ALPHA   ));
append(  new Glyph( 'N',   KC_38,  SHIFTED,    RH_RING,    SHIFTED_ALPHA   ));
append(  new Glyph( 'S',   KC_39,  SHIFTED,    RH_PINKY,   SHIFTED_ALPHA   ));
append(  new Glyph( 'Q',   KC_45,  SHIFTED,    LH_RING,    SHIFTED_ALPHA   ));
append(  new Glyph( 'J',   KC_46,  SHIFTED,    LH_MIDDLE,  SHIFTED_ALPHA   ));
append(  new Glyph( 'K',   KC_47,  SHIFTED,    LH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'X',   KC_48,  SHIFTED,    LH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'B',   KC_49,  SHIFTED,    RH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'M',   KC_50,  SHIFTED,    RH_INDEX,   SHIFTED_ALPHA   ));
append(  new Glyph( 'W',   KC_51,  SHIFTED,    RH_MIDDLE,  SHIFTED_ALPHA   ));
append(  new Glyph( 'V',   KC_52,  SHIFTED,    RH_RING,    SHIFTED_ALPHA   ));
append(  new Glyph( 'Z',   KC_53,  SHIFTED,    RH_PINKY,   SHIFTED_ALPHA   ));

append(  new Glyph( 'p',   KC_19,  UNSHIFTED,  LH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'y',   KC_20,  UNSHIFTED,  LH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'f',   KC_21,  UNSHIFTED,  RH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'g',   KC_22,  UNSHIFTED,  RH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'c',   KC_23,  UNSHIFTED,  RH_MIDDLE,  UNSHIFTED_ALPHA ));
append(  new Glyph( 'r',   KC_24,  UNSHIFTED,  RH_RING,    UNSHIFTED_ALPHA ));
append(  new Glyph( 'l',   KC_25,  UNSHIFTED,  RH_PINKY,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'a',   KC_30,  UNSHIFTED,  LH_PINKY,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'o',   KC_31,  UNSHIFTED,  LH_RING,    UNSHIFTED_ALPHA ));
append(  new Glyph( 'e',   KC_32,  UNSHIFTED,  LH_MIDDLE,  UNSHIFTED_ALPHA ));
append(  new Glyph( 'u',   KC_33,  UNSHIFTED,  LH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'i',   KC_34,  UNSHIFTED,  LH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'd',   KC_35,  UNSHIFTED,  RH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'h',   KC_36,  UNSHIFTED,  RH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 't',   KC_37,  UNSHIFTED,  RH_MIDDLE,  UNSHIFTED_ALPHA ));
append(  new Glyph( 'n',   KC_38,  UNSHIFTED,  RH_RING,    UNSHIFTED_ALPHA ));
append(  new Glyph( 's',   KC_39,  UNSHIFTED,  RH_PINKY,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'q',   KC_45,  UNSHIFTED,  LH_RING,    UNSHIFTED_ALPHA ));
append(  new Glyph( 'j',   KC_46,  UNSHIFTED,  LH_MIDDLE,  UNSHIFTED_ALPHA ));
append(  new Glyph( 'k',   KC_47,  UNSHIFTED,  LH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'x',   KC_48,  UNSHIFTED,  LH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'b',   KC_49,  UNSHIFTED,  RH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'm',   KC_50,  UNSHIFTED,  RH_INDEX,   UNSHIFTED_ALPHA ));
append(  new Glyph( 'w',   KC_51,  UNSHIFTED,  RH_MIDDLE,  UNSHIFTED_ALPHA ));
append(  new Glyph( 'v',   KC_52,  UNSHIFTED,  RH_RING,    UNSHIFTED_ALPHA ));
append(  new Glyph( 'z',   KC_53,  UNSHIFTED,  RH_PINKY,   UNSHIFTED_ALPHA ));
	}
}
