//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 1999  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// Typist class - creates the main program and links everything
// together
//
// Example applet tag:
// <APPLET CODE="Typist" WIDTH=100 HEIGHT=50>
// 	<PARAM NAME="locale" VALUE="en_US">
// 	<PARAM NAME="kbtype" VALUE="US">
// 	<PARAM NAME="series" VALUE="T,R,Q,V,U,D,M,S,N">
// 	<PARAM NAME="buttonlabel" VALUE="Run Typist">
// </APPLET>
//
// Add a <PARAM NAME="realkbtype" VALUE="xx"> entry to enable keymap
// translations.  Example:
// 	<PARAM NAME="kbtype" VALUE="Dvorak">
//	<PARAM NAME="realkbtype" VALUE="US">
//
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.StringTokenizer;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.ListResourceBundle;
import java.text.MessageFormat;

/**
 * This is the main class for the Typist tutor.
 */
public class Typist extends Applet
		implements ActionListener {

	// message catalog information
	private ListResourceBundle	messages	= null;

	// default argument values
	public static final Locale	DEFAULT_LOCALE	= Locale.getDefault();
	public static final String	DEFAULT_KBTYPE	= "US";
	public static final String	DEFAULT_REALKBTYPE
							= "US";
	public static final String	DEFAULT_SERIES	= "T,R,Q,V,U,D,M,S,N";
	public static final String	DEFAULT_BUTTONLABEL
							= "Run Typist";

	// keyboard type to use, run button text, and command line
	// flags for the application form of the program
	private Locale	locale		= DEFAULT_LOCALE;
							// application/applet
	private String	kbtype		= DEFAULT_KBTYPE;
							// application/applet
	private String	real_kbtype	= DEFAULT_REALKBTYPE;
							// application/applet
	private String	series		= DEFAULT_SERIES;
							// application/applet
	private String	buttonlabel	= DEFAULT_BUTTONLABEL;
							// applet
	private boolean	lflag		= false;	// application/applet
	private boolean	kflag		= false;	// application/applet
	private boolean	rflag		= false;	// application/applet
	private boolean	sflag		= false;	// application/applet
	private boolean	mflag		= false;	// application/applet
	private boolean	dflag		= false;	// application
	private boolean	hflag		= false;	// application
	private boolean	vflag		= false;	// application

	// keyboard layout, created from the kbtype passed in
	private KBLayout	kblayout	= null;

	// translation keyboard layout, created from the realkbtype passed in
	private KBLayout	real_kblayout	= null;

	// a GUI that indicate startup progress
	private InitProgressGUI	initprogress	= null;


	/**
	 * Constructor to initialize a default locale message catalog.
	 * <p>
	 * The catalog resource may be replaced by a new one if a locale is
	 * specified to the program by options or applet parameters.
	 */
	public Typist() {

		// initialize messages to the default locale
		messages = (ListResourceBundle)
				(ResourceBundle.getBundle(
					"MessageResource", locale ));
	}


	/**
	 * Convenience to return a locale from a nn_CC[_vvvv...] string.
	 */
	private Locale parse_locale( String lstring ) {

		// tokenize the string on underscores
		StringTokenizer t = new StringTokenizer( lstring, "_" );

		// if three parts, return the suitable locale
		if ( t.countTokens() >= 3 )
			return ( new Locale( t.nextToken(), t.nextToken(),
					t.nextToken() ));

		// if only two parts return that locale
		else if ( t.countTokens() == 2 )
			return ( new Locale( t.nextToken(), t.nextToken() ));

		// if only one parts then treat that as the language
		else if ( t.countTokens() == 1 )
			return ( new Locale( t.nextToken(), "" ));

		// if nothing visible, just return the default locale
		else
			return ( Locale.getDefault() );
	}

	/*
	 * Parse command line arguments into storage for the application.
	 */
	private void parse_args( String args[] ) {
		for ( int index = 0; index < args.length; index++ ) {
			if ( args[ index ].equals( "-l" )
					|| args[ index ].equals( "-locale" )) {
				lflag = true;
				if ( index < args.length - 1 ) {
					locale = parse_locale( args[ ++index ] );
					messages = (ListResourceBundle)
						(ResourceBundle.getBundle(
						"MessageResource", locale ));
				}
				else {
					String[] fmtargs = { args[ index ] };
					System.err.println(
					MessageFormat.format(
						messages.getString(
							"Typist.missingarg" ),
							fmtargs ));
					System.err.println(
						messages.getString(
							"Typist.tryusage" ) );
					System.exit( 1 );
				}
			}
			else if ( args[ index ].equals( "-k" )
					|| args[ index ].equals( "-kb" )
					|| args[ index ].equals( "-kbtype" )) {
				kflag = true;
				if ( index < args.length - 1 ) {
					kbtype = args[ ++index ];
					kblayout = (KBLayout)ObjectFactory.create_object(
							"KBLayout"  + kbtype );
					if ( kblayout == null ) {
						String[] fmtargs = { kbtype };
						System.err.println(
						MessageFormat.format(
							messages.getString(
							"Typist.invalidkb" ),
							 	fmtargs ));
						System.err.println(
							messages.getString(
							"Typist.tryusage" ) );
						System.exit( 1 );
					}
				}
				else {
					String[] fmtargs = { args[ index ] };
					System.err.println(
					MessageFormat.format(
						messages.getString(
							"Typist.missingarg" ),
							fmtargs ));
					System.err.println(
						messages.getString(
							"Typist.tryusage" ) );
					System.exit( 1 );
				}
			}
			else if ( args[ index ].equals( "-r" )
					|| args[ index ].equals( "-realkb" )
					|| args[ index ].equals( "-realkbtype" )) {
				rflag = true;
				if ( index < args.length - 1 ) {
					real_kbtype = args[ ++index ];
					real_kblayout = (KBLayout)ObjectFactory.create_object(
							"KBLayout"  + real_kbtype );
					if ( real_kblayout == null ) {
						String[] fmtargs = {
								real_kbtype };
						System.err.println(
						MessageFormat.format(
							messages.getString(
							"Typist.invalidkb" ),
							 	fmtargs ));
						System.err.println(
							messages.getString(
							"Typist.tryusage" ) );
						System.exit( 1 );
					}
				}
				else {
					String[] fmtargs = { args[ index ] };
					System.err.println(
					MessageFormat.format(
						messages.getString(
							"Typist.missingarg" ),
							fmtargs ));
					System.err.println(
						messages.getString(
							"Typist.tryusage" ) );
					System.exit( 1 );
				}
			}
			else if ( args[ index ].equals( "-s" )
					|| args[ index ].equals( "-series" )) {
				sflag = true;
				if ( index < args.length - 1 ) {
					series = args[ ++index ];
					if ( series.length() == 0 ) {
						System.err.println(
							messages.getString(
							"Typist.noseries" ));
						System.err.println(
							messages.getString(
							"Typist.tryusage" ) );
						System.exit( 1 );
					}
				}
				else {
					String[] fmtargs = { args[ index ] };
					System.err.println(
					MessageFormat.format(
						messages.getString(
							"Typist.missingarg" ),
							fmtargs ));
					System.err.println(
						messages.getString(
							"Typist.tryusage" ) );
					System.exit( 1 );
				}
			}
			else if ( args[ index ].equals( "-m" )
					|| args[ index ].equals( "-msgs" )
					|| args[ index ].equals( "-messages" ))
				mflag = true;
			else if ( args[ index ].equals( "-d" )
					|| args[ index ].equals( "-doc" )
					|| args[ index ].equals( "-documentation" ))
				dflag = true;
			else if ( args[ index ].equals( "-h" )
					|| args[ index ].equals( "-help" ))
				hflag = true;
			else if ( args[ index ].equals( "-v" )
					|| args[ index ].equals( "-version" ))
				vflag = true;
			else {
				String[] fmtargs = { args[ index ] };
				System.err.println(
					MessageFormat.format(
						messages.getString(
							"Typist.illegalopt" ),
							fmtargs ));
				System.err.println(
					messages.getString(
							"Typist.tryusage" ) );
				System.exit( 1 );
			}
		}

		// if no kbtype was given then default it
		if ( kblayout == null )
			kblayout = (KBLayout)ObjectFactory.create_object(
					"KBLayout" + kbtype );
	}


	/**
	 * Initialize the applet version of the program.
	 * <p>
	 * This is the main entry point for the Typist applet.
	 */
	public void init() {

		// get the parameters - if a locale is given, hop to it;
		// also, deal with the locale first, since this way if there
		// are errors in what follows, the messages are localized
		String lstring = getParameter( "locale" );
		if ( lstring != null ) {
			lflag = true;
			locale = parse_locale( lstring );
			messages = (ListResourceBundle)
				(ResourceBundle.getBundle(
					"MessageResource", locale ));
		}
		// if no kbtype or invalid, just default to initialized
		// value (US)
		String param = getParameter( "kbtype" );
		if ( param != null ) {
			kflag = true;
			kblayout = (KBLayout)ObjectFactory.create_object(
					"KBLayout" + param );
			if ( kblayout != null )
				kbtype = param;
			else {
				System.out.println(
					messages.getString(
						"Typist.invalidkbtype" ));
				String[] fmtargs = { kbtype };
				System.out.println(
					MessageFormat.format(
						messages.getString(
							"Typist.defaultingto" ),
							fmtargs ));
				kblayout = (KBLayout)ObjectFactory.create_object(
						"KBLayout" + kbtype );
			}
		}
		// if no realkbtype or invalid, don't instantiate anything
		param = getParameter( "realkbtype" );
		if ( param != null ) {
			rflag = true;
			real_kblayout = (KBLayout)ObjectFactory.create_object(
					"KBLayout" + param );
			if ( real_kblayout != null )
				real_kbtype = param;
			else {
				System.out.println(
					messages.getString(
						"Typist.invalidrealkbtype" ));
				System.out.println(
					messages.getString(
						"Typist.parameterignored" ));
			}
		}
		param = getParameter( "buttonlabel" );
		if ( param != null )
			buttonlabel = param;
		else
			buttonlabel = messages.getString(
					"Typist.buttonlabel" );
		param = getParameter( "series" );
		if ( param != null && param.length() > 0 ) {
			sflag = true;
			series = param;
		}
		param = getParameter( "messages" );
		if ( param != null )
			mflag = true;

		// if no kbtype was given then default it
		if ( kblayout == null )
			kblayout = (KBLayout)ObjectFactory.create_object(
					"KBLayout" + kbtype );

		// create a get-it-all-going button as the only part
		// of the applet - necessary since the main code makes
		// heavy use of frames and menubars
		Button run_button = new Button( buttonlabel );
		run_button.addActionListener( this );
		setLayout( new FlowLayout( FlowLayout.CENTER, 0, 0 ) );
		add( run_button );
	}


	/*
	 * Convenience function to show some startup status where the
	 * run up to getting the display on the screen is a slow one.
	 */
	private void show_status( boolean inAnApplet, String status ) {

		// expected number of checkpoint calls that we will make
		// to the init progress indicator, based on how many
		// calls we get
		final int INIT_CHECKPOINTS	= 10;

		// if this is the first call then put up a 'starting up' frame,
		// the wait a while to let it render
		if ( initprogress == null ) {
			initprogress = new InitProgressGUI( inAnApplet,
				locale,
				INIT_CHECKPOINTS,
				messages.getString( "Typist.starting" ));
			initprogress.pack();
			initprogress.setVisible( true );
			try { Thread.sleep( 200 ); }
			catch ( InterruptedException e ) {}
		}

		// checkpoint the progress indicator
		initprogress.checkpoint();
		try { Thread.sleep( 200 ); }
		catch ( InterruptedException e ) {}

		// finally just print out the message to stdout
		if ( mflag )
			if ( inAnApplet )
				showStatus( status );
			else
				System.out.println( status );
	}


	/*
	 * Start up all the frames of the real program - called by either
	 * the action routine of the applet's run button, or by main in
	 * the application.
	 */
	private void start_frames( boolean inAnApplet ) {

		// create the control window, but keep it hidden for
		// the moment
		// print out the locale as a confirmation
		String[] fmtargs = { locale.getDisplayName( locale ) };
		show_status( inAnApplet,
			MessageFormat.format(
				messages.getString( "Typist.controlgui" ),
								fmtargs ));
		ControlGUI control = new ControlGUI( inAnApplet, locale );

		// add lesson series to the control window
		show_status( inAnApplet,
			messages.getString( "Typist.addingseries" ));
		StringTokenizer st = new StringTokenizer( series, " \t,:" );
		while ( st.hasMoreTokens() ) {
			String series_id = st.nextToken();
			control.add_series( series_id );
		}

		// create the sub-windows
		show_status( inAnApplet,
			messages.getString( "Typist.tutorialgui" ));
		TutorialGUI tutorial = new TutorialGUI( inAnApplet, locale );
		tutorial.pack(); tutorial.setVisible( false );

		String fmtargs2[] = { kbtype };
		show_status( inAnApplet, MessageFormat.format(
			messages.getString( "Typist.kbstatistics" ),
								fmtargs2 ));
		KBStatistics kbstats = new KBStatistics( kblayout );
		show_status( inAnApplet,
			messages.getString( "Typist.statisticsgui" ));
		StatisticsGUI statistics = new StatisticsGUI( inAnApplet,
				kbstats, locale );
		statistics.pack(); statistics.setVisible( false );

		show_status( inAnApplet,
			messages.getString( "Typist.keyboardgui" ));
		KeyboardGUI keyboard = new KeyboardGUI( inAnApplet,
				kblayout, locale );
		keyboard.pack(); keyboard.setVisible( false );

		show_status( inAnApplet,
			messages.getString( "Typist.exercisegui" ));
		ExerciseGUI exercise = new ExerciseGUI( inAnApplet,
				kbstats, locale );
		exercise.pack(); exercise.setVisible( false );

		show_status( inAnApplet,
			messages.getString( "Typist.helpgui" ));
		HelpGUI help = new HelpGUI( inAnApplet, locale );
		help.pack(); help.setVisible( false );

		show_status( inAnApplet,
			messages.getString( "Typist.aboutgui" ));
		AboutGUI about = new AboutGUI( inAnApplet, control, false,
				locale );
		about.pack(); about.setVisible( false );

		// register all the sub-windows with the control window
		show_status( inAnApplet,
			messages.getString( "Typist.linking" ));
		exercise.register_statistics( statistics );
		exercise.register_keyboard( keyboard );
		control.register_about( about );
		control.register_tutorial( tutorial );
		control.register_exercise( exercise );
		control.register_statistics( statistics );
		control.register_keyboard( keyboard );
		control.register_help( help );

		// if we have to do keymap translations, add a translator to
		// the exercise GUI
		if ( real_kblayout != null )
			exercise.register_translator(
				new KBTranslator( real_kblayout, kblayout ));

		// finally reveal the control window
		show_status( inAnApplet,
			messages.getString( "Typist.initialized" ));
		control.pack(); control.setVisible( true );

		// if in applet mode, clear the status line
		if ( inAnApplet )
			show_status( inAnApplet, "" );

		// dispose of any initialization progress indicator
		if ( initprogress != null ) {
			initprogress.dispose();
			initprogress = null;
		}
	}


	/*
 	 * Handle the GUI actions in the applet.
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof Button ) {

			// must be the run button, so start up the
			// frames and stuff, then sit back and watch
			// the antics
			start_frames( true );
		}
	}


	/**
	 * Parse the command line arguments, create frames, and run
	 * the application.
	 * <p>
	 * This is the main entry point for the Typist application.
	 */
	public static void main( String args[] ) {

		// parse the arguments
		Typist typist = new Typist();
		typist.parse_args( args );

		// do the easy stuff with doc, help and version first
		if ( typist.dflag ) {
			System.out.println(
				HelpHTML.get_HTML_helptext( typist.locale ));
			System.exit( 0 );
		}
		if ( typist.hflag ) {
			String[] fmtargs = {
				typist.DEFAULT_KBTYPE,
				typist.DEFAULT_SERIES,
				typist.DEFAULT_LOCALE.toString()
			};
			System.err.println(
				MessageFormat.format(
					typist.messages.getString(
						"Typist.usage" ), fmtargs ));
			System.exit( 0 );
		}
		if ( typist.vflag ) {
			System.err.println(
				typist.messages.getString( "Typist.version" ));
			System.exit( 0 );
		}

		// create the frames set (no need for the intermediate
		// applet "run" button here)
		typist.start_frames( false );
	}
}
